"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BulkOperationEnum = exports.FirestoreCollection = exports.FirestoreClient = exports.FirestoreUtil = void 0;
const admin = __importStar(require("firebase-admin"));
const lodash_chunk_1 = __importDefault(require("lodash.chunk"));
const MAX_BATCH_SIZE = 500;
class FirestoreUtil {
    static decode(documentSnapshot) {
        return Object.assign(Object.assign({}, this.timestampsToDates(documentSnapshot.data())), { created: documentSnapshot.createTime.toDate(), lastModified: documentSnapshot.updateTime.toDate(), id: documentSnapshot.id });
    }
    static timestampsToDates(data) {
        Object.keys(data).forEach(key => {
            if (data[key] && typeof data[key] === 'object' && (data[key]["_seconds"] !== undefined && data[key]["_nanoseconds"] !== undefined))
                data[key] = data[key].toDate();
            else if (data[key] && typeof data[key] === 'object') {
                this.timestampsToDates(data[key]);
            }
        });
        return data;
    }
    static datesToTimestamps(data) {
        Object.keys(data).forEach(key => {
            if (data[key] && typeof data[key] === 'object' && data[key] instanceof Date) {
                data[key] = admin.firestore.Timestamp.fromDate(data[key]);
            }
            else if (data[key] && typeof data[key] === 'object') {
                data[key] = this.datesToTimestamps(data[key]);
            }
            else if (data[key] && typeof data[key] === 'string') {
                const parsedDate = this.parseIsoDate(data[key]);
                if (parsedDate != undefined) {
                    data[key] = admin.firestore.Timestamp.fromDate(parsedDate);
                }
            }
        });
        return data;
    }
    static parseIsoDate(input) {
        if (!/\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}.\d{3}Z/.test(input) &&
            !/\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}Z/.test(input))
            return undefined;
        return new Date(input);
    }
}
exports.FirestoreUtil = FirestoreUtil;
const MAX_WHERE_IN = 10;
class FirestoreClient {
    constructor(firestore) {
        this.firestore = firestore;
    }
    getCollection(collectionName, optionalFields = [], versioned = true) {
        return new FirestoreCollection(this.firestore.collection(collectionName), this.firestore, optionalFields, versioned);
    }
    runTransaction(updateFunction, transactionOptions) {
        return this.firestore.runTransaction(updateFunction, transactionOptions);
    }
}
exports.FirestoreClient = FirestoreClient;
class FirestoreCollection {
    constructor(collectionReference, firestore, optionalFields, versioned = true) {
        this.collectionReference = collectionReference;
        this.firestore = firestore;
        this.optionalFields = optionalFields;
        this.versioned = versioned;
    }
    async documentExists(id, transaction) {
        const docRef = typeof id === "string" ? this.collectionReference.doc(id) : id;
        const documentSnapshot = transaction ? await transaction.get(docRef) : await docRef.get();
        if (documentSnapshot === undefined || !documentSnapshot.exists) {
            return Promise.resolve(false);
        }
        return Promise.resolve(true);
    }
    async getDocument(id, transaction) {
        const docRef = typeof id === "string" ? this.collectionReference.doc(id) : id;
        const documentSnapshot = transaction ? await transaction.get(docRef) : await docRef.get();
        if (documentSnapshot === undefined || !documentSnapshot.exists) {
            return Promise.resolve(undefined);
        }
        return Promise.resolve(FirestoreUtil.decode(documentSnapshot));
    }
    getSubCollection(documentId, collectionName, optionalFields = [], versioned = true) {
        this.collectionReference.doc(documentId).collection(collectionName);
        return new FirestoreCollection(this.collectionReference.doc(documentId).collection(collectionName), this.firestore, optionalFields, versioned);
    }
    async setDocument(id, data) {
        this.deleteOptionalFields(data);
        if (this.versioned)
            this.applyVersionIncrement(FirestoreUtil.datesToTimestamps(data));
        const docRef = this.collectionReference.doc(id);
        return docRef.set(data, { merge: true });
    }
    async listDocuments() {
        const docRefs = await this.collectionReference.listDocuments();
        if (docRefs !== undefined) {
            return Promise.all(docRefs.map(async (doc) => FirestoreUtil.decode(await doc.get())));
        }
        return Promise.reject();
    }
    async addDocument(data) {
        if (this.versioned)
            this.applyVersionIncrement(data);
        const docRef = await this.collectionReference.add(FirestoreUtil.datesToTimestamps(data));
        if (docRef === undefined) {
            return Promise.reject();
        }
        return this.getDocument(docRef);
    }
    async updateDocument(id, updateData, transaction) {
        if (this.versioned)
            this.applyVersionIncrement(updateData);
        const docRef = this.collectionReference.doc(id);
        transaction ? await transaction.update(docRef, FirestoreUtil.datesToTimestamps(updateData)) : await docRef.update(updateData);
        return this.getDocument(id);
    }
    async deleteDocument(id) {
        return this.collectionReference.doc(id).delete();
    }
    setDocumentInTransaction(id, data, transaction) {
        this.deleteOptionalFields(data);
        if (this.versioned)
            this.applyVersionIncrement(data);
        let docRef;
        if (!id) {
            docRef = this.collectionReference.doc();
        }
        else {
            docRef = typeof id === "string" ? this.collectionReference.doc(id) : id;
        }
        return transaction.set(docRef, FirestoreUtil.datesToTimestamps(data), { merge: true });
    }
    async executeQuery(fieldPath, opStr, value) {
        const querySnapshot = await this.collectionReference.where(fieldPath, opStr, value).get();
        if (querySnapshot !== undefined) {
            return querySnapshot.docs.map(doc => FirestoreUtil.decode(doc));
        }
        return Promise.reject();
    }
    startQuery(fieldPath, opStr, value) {
        return this.collectionReference.where(fieldPath, opStr, value);
    }
    async getQueryResult(query) {
        const querySnapshot = await query.get();
        if (querySnapshot !== undefined) {
            return querySnapshot.docs.map(doc => FirestoreUtil.decode(doc));
        }
        return Promise.reject();
    }
    async bulkSet(data) {
        await Promise.all(lodash_chunk_1.default(data, MAX_BATCH_SIZE).map(async (dataRefs) => {
            const writeBatch = this.firestore.batch();
            (dataRefs).map(value => {
                this.deleteOptionalFields(value);
                if (this.versioned)
                    this.applyVersionIncrement(value);
                const docRef = value.id ? this.collectionReference.doc(value.id) : this.collectionReference.doc();
                writeBatch.set(docRef, FirestoreUtil.datesToTimestamps(value), { merge: true });
            });
            await writeBatch.commit();
        })).catch(reason => {
            return Promise.reject();
        });
    }
    async bulkModification(data) {
        await Promise.all(lodash_chunk_1.default(data, MAX_BATCH_SIZE).map(async (bulkOperations) => {
            const writeBatch = this.firestore.batch();
            (bulkOperations).map(bulkOperation => {
                const docRef = bulkOperation.payloadId ? this.collectionReference.doc(bulkOperation.payloadId) : this.collectionReference.doc();
                switch (bulkOperation.operation) {
                    case BulkOperationEnum.CREATE:
                    case BulkOperationEnum.UPDATE:
                        this.deleteOptionalFields(bulkOperation.payload);
                        if (this.versioned)
                            this.applyVersionIncrement(bulkOperation.payload);
                        writeBatch.set(docRef, FirestoreUtil.datesToTimestamps(bulkOperation.payload), { merge: true });
                        break;
                    case BulkOperationEnum.DELETE:
                        writeBatch.delete(this.collectionReference.doc(bulkOperation.payloadId));
                        break;
                    default:
                        break;
                }
            });
            await writeBatch.commit();
        })).catch(reason => {
            return Promise.reject();
        });
    }
    async getDocumentsById(ids, transaction) {
        let result = [];
        await Promise.all(lodash_chunk_1.default(ids, MAX_WHERE_IN).map(async (idsChunk) => {
            const query = this.collectionReference.where(admin.firestore.FieldPath.documentId(), "in", idsChunk);
            const queryResult = await (transaction ? transaction.get(query) : query.get());
            result = result.concat(queryResult.docs.map(value => FirestoreUtil.decode(value)));
        }));
        return result;
    }
    applyVersionIncrement(data) {
        data.version = admin.firestore.FieldValue.increment(1);
    }
    deleteOptionalFields(data) {
        this.optionalFields
            .filter(prop => data[prop] === undefined)
            .map(value => data[value] = admin.firestore.FieldValue.delete());
    }
}
exports.FirestoreCollection = FirestoreCollection;
var BulkOperationEnum;
(function (BulkOperationEnum) {
    BulkOperationEnum[BulkOperationEnum["CREATE"] = 'CREATE'] = "CREATE";
    BulkOperationEnum[BulkOperationEnum["UPDATE"] = 'UPDATE'] = "UPDATE";
    BulkOperationEnum[BulkOperationEnum["DELETE"] = 'DELETE'] = "DELETE";
})(BulkOperationEnum = exports.BulkOperationEnum || (exports.BulkOperationEnum = {}));
//# sourceMappingURL=FirestoreClient.js.map